<?php

namespace Modules\Fm\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Modules\Fm\Entities\SupplierInvoiceAttachment;
use Modules\Fm\Entities\SupplierInvoiceData;
use Modules\Fm\Entities\SupplierInvoiceHeader;
use Modules\Fm\Entities\SupplierInvoiceOther;
use Modules\Sc\Entities\GrnData;
use Modules\Sc\Entities\GrnHeader;
use Modules\Sc\Entities\ProductHistory;
use Modules\Sc\Entities\ProductHistorySetoff;
use Modules\St\Entities\Product;
use Modules\St\Entities\ProductType;
use Modules\St\Entities\ProductUnit;
use Modules\St\Entities\Suppliers;

class SupplierInvoiceController extends Controller
{
    public function allProduct()
    {
        try {
            $products =  Product::select('product_primary_id As id', 'product_id As value0', 'product_name As value1')->get();
            return $this->responseBody(true, "SupplierInvoiceController", "allProduct", $products);
        } catch (Exception $exception) {
            return $this->responseBody(false, "SupplierInvoiceController", "error", $exception);
        }
    }



    public function allSuppliers()
    {
        try {
            $products =  Suppliers::select('supplier_id As id', 'supplier_name As value0')->get();
            return $this->responseBody(true, "SupplierInvoiceController", "allSuppliers", $products);
        } catch (Exception $exception) {
            return $this->responseBody(false, "SupplierInvoiceController", "error", $exception);
        }
    }




    public function referanceMaxID()
    {
        try {
            //$store = StoreLocation::find($store_id);
            $id = 0;
            $prefix = "";
            $result = DB::select('SELECT MAX(external_id) AS id FROM fm_supplier_invoice_headers');
            foreach ($result as $res) {
                if ($res->id) {
                    $id = $res->id;
                    $id = preg_replace('/\D/', '', $id);
                }

                $id += 1;
            }
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "grnHeaderMaxID", ['id' => $id, 'prefix' => $prefix]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }





    public function save(Request $request)
    {
        try {
            $prefix = ""; //$this->getPrefix('100');

            $header = new SupplierInvoiceHeader();
            $header->external_id = $prefix . "" . $request->get('external_id');
            $header->ref_external_id = $prefix . "" . $request->get('ref_extenal_id');
            $header->trans_date = $request->get('trans_date');
            $header->supplier_Id = $request->get('supplier_id');
            $header->supplier_ref_no = $request->get('supplier_ref_no');
            $header->document_no = "";
            $header->user_id = Auth::user()->id;
            $header->Status = 0;
            if ($header->save()) {
                $header_find = SupplierInvoiceHeader::find($header->internal_id);
                $header_find->ref_internal_id = $header->internal_id;
                $header_find->update();
                $invoice_data_arr = $request->get("supplier_invoice_data");
                for ($i = 0; $i < count($invoice_data_arr); $i++) {
                    $invoiceData = json_decode($invoice_data_arr[$i]);
                    $data = new SupplierInvoiceData();
                    $data->internal_id = $header_find->internal_id;
                    $data->external_id = $header_find->external_id;
                    $data->grn_data_id = $invoiceData->grn_data_id;
                    $data->grn_internal_id = $invoiceData->grn_internal_id;
                    $data->grn_external_id = $invoiceData->grn_external_id;
                    $data->product_id =  $invoiceData->product_id;
                    $data->unit_id =  $invoiceData->unit_id;
                    $data->quantity =  $invoiceData->qty;
                    $data->price =  $invoiceData->price;
                    $data->discount_id =  $invoiceData->discount_id;
                    $data->discount =  $invoiceData->discount;
                    $data->value =  $invoiceData->value;
                    if ($data->save()) {
                        $grn_data = GrnData::find($invoiceData->grn_data_id);
                        if ($grn_data) {
                            $grn_data->price = $invoiceData->price;
                            $grn_data->update();
                        }
                    }
                }



                $supplier_invoice_other_arr = $request->get("supplier_invoice_other");
                if ($supplier_invoice_other_arr) {
                    for ($i = 0; $i < count($supplier_invoice_other_arr); $i++) {
                        $description = $supplier_invoice_other_arr[$i]["description"];
                        $qty = $supplier_invoice_other_arr[$i]["quantity"];
                        $price = $supplier_invoice_other_arr[$i]["price"];
                        $discount = $supplier_invoice_other_arr[$i]["discount"];
                        $value = $supplier_invoice_other_arr[$i]["value"];

                        if ($description == null) {
                            $description = "";
                        }
                        if ($qty == null) {
                            $qty = "";
                        }
                        if ($price == null) {
                            $price = "";
                        }
                        if ($discount == null) {
                            $discount = "";
                        }
                        if ($value == null) {
                            $value = "";
                        }
                        $sup_other = new SupplierInvoiceOther();
                        $sup_other->internal_id = $header->internal_id;
                        $sup_other->external_id = $header->external_id;
                        $sup_other->description = $description;
                        $sup_other->quantity = $qty;
                        $sup_other->price =  $price;
                        $sup_other->discount =  $discount;
                        $sup_other->value =  $value;
                        $sup_other->save();
                    }
                }


                $attachmnet_token = $request->get("attachment_token");
                $attachments = SupplierInvoiceAttachment::where('token', '=', $attachmnet_token)->get();
                foreach ($attachments as $attachmnet) {
                    if ($attachmnet) {
                        $attachmnet->supplier_invoice_header_id = $header->internal_id;
                        $attachmnet->update();
                    }
                }
            }
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "save", $invoice_data_arr);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    public function uploadAttachment(Request $request)
    {

        $id = $request->get('supplier_invoice_id');
        $token = $request->get('attachmnet_token');
        $file = $request->file('file');
        $file_name = $file->getClientOriginalName();
        $filename = uniqid() . '_' . time() . '.' . $file_name;
        $filename = str_replace(' ', '', str_replace('\'', '', $filename));
        $file->move(public_path('finance'), $filename);

        $invoice_attachment = new SupplierInvoiceAttachment();
        $invoice_attachment->supplier_invoice_header_id = 0;
        $invoice_attachment->path = $filename;
        $invoice_attachment->token = $token;
        return $invoice_attachment->save();
    }




    public function allAttachment($id, $token)
    {
        try {

            $attachment = DB::select('SELECT *FROM fm_supplier_invoice_attachments WHERE  supplier_invoice_header_id = "' . $id . '" OR token = "' . $token . '"');
            $responseBody = $this->responseBody(true, "allAttachment", "found", $attachment);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "allAttachment", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function getSupplierInvoice($id)
    {
        try {
            $grn = SupplierInvoiceHeader::find($id);
            if (count(array_filter(explode("-", $grn->external_id))) == 2) {
                $grn->external_id = explode("-", $grn->external_id)[1];
            }
            $supplier = Suppliers::find($grn->supplier_Id);
            //$received = Employee::find($grn->received_by);
            //$grn->received = $received->name_withinitial;
            $grn->supplier = $supplier->supplier_name;
            $grn_data = SupplierInvoiceData::where('internal_id', '=', $id)->get();
            foreach ($grn_data as $data) {
                $product = Product::find($data->product_id);
                $data->product = $product->product_name;
                $uom = ProductUnit::find($data->unit_id);
                $uom_arr = [];
                if ($uom) {
                    array_push($uom_arr, ["value" => $uom->id, "text" => $uom->name]);
                }
                $discount_arr = [];
                array_push($discount_arr, ["value" => "1", "text" => "%"]);
                array_push($discount_arr, ["value" => "2", "text" => "Amount"]);
                $data->uom = $uom_arr;
                $data->discount_type = $discount_arr;
                $data->product_primary_id = $product->product_primary_id;
                $data->product_id = $product->product_id;
            }
            $grn->data = $grn_data;
            $invoice_other = SupplierInvoiceOther::where('internal_id', '=', $id)->get();
            $grn->request_other = $invoice_other;
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "getSupplierInvoice", $grn);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "getSupplierInvoice", $id);
        }
        return response()->json(["data" => $responseBody]);
    }




    public function update(Request $request, $id)
    {
        try {
            $prefix = ""; //$this->getPrefix('100');

            if ($this->delete($id)) {
                $header = SupplierInvoiceHeader::find($id);
                $header->external_id = $prefix . "" . $request->get('external_id');
                $header->ref_external_id = $prefix . "" . $request->get('ref_extenal_id');
                $header->trans_date = $request->get('trans_date');
                $header->supplier_Id = $request->get('supplier_id');
                $header->supplier_ref_no = $request->get('supplier_ref_no');
                $header->document_no = "";
                $header->user_id = Auth::user()->id;
                // $header->Status = 0;
                if ($header->update()) {
                    $invoice_data_arr = $request->get("supplier_invoice_data");
                    for ($i = 0; $i < count($invoice_data_arr); $i++) {
                        $invoiceData = json_decode($invoice_data_arr[$i]);
                        $data = new SupplierInvoiceData();
                        $data->internal_id = $header->internal_id;
                        $data->external_id = $header->external_id;
                        $data->grn_data_id = $invoiceData->grn_data_id;
                        $data->grn_internal_id = $invoiceData->grn_internal_id;
                        $data->grn_external_id = $invoiceData->grn_external_id;
                        $data->product_id =  $invoiceData->product_id;
                        $data->unit_id =  $invoiceData->unit_id;
                        $data->quantity =  $invoiceData->qty;
                        $data->price =  $invoiceData->price;
                        $data->discount_id =  $invoiceData->discount_id;
                        $data->discount =  $invoiceData->discount;
                        $data->value =  $invoiceData->value;
                        $data->save();
                    }


                    $supplier_invoice_other_arr = $request->get("supplier_invoice_other");
                    if ($supplier_invoice_other_arr) {
                        for ($i = 0; $i < count($supplier_invoice_other_arr); $i++) {
                            $description = $supplier_invoice_other_arr[$i]["description"];
                            $qty = $supplier_invoice_other_arr[$i]["quantity"];
                            $price = $supplier_invoice_other_arr[$i]["price"];
                            $discount = $supplier_invoice_other_arr[$i]["discount"];
                            $value = $supplier_invoice_other_arr[$i]["value"];

                            if ($description == null) {
                                $description = "";
                            }
                            if ($qty == null) {
                                $qty = "";
                            }
                            if ($price == null) {
                                $price = "";
                            }
                            if ($discount == null) {
                                $discount = "";
                            }
                            if ($value == null) {
                                $value = "";
                            }
                            $sup_other = new SupplierInvoiceOther();
                            $sup_other->internal_id = $header->internal_id;
                            $sup_other->external_id = $header->external_id;
                            $sup_other->description = $description;
                            $sup_other->quantity = $qty;
                            $sup_other->price =  $price;
                            $sup_other->discount =  $discount;
                            $sup_other->value =  $value;
                            $sup_other->save();
                        }
                    }


                    $attachmnet_token = $request->get("attachment_token");
                    $attachments = SupplierInvoiceAttachment::where('token', '=', $attachmnet_token)->get();
                    foreach ($attachments as $attachmnet) {
                        if ($attachmnet) {
                            $attachmnet->supplier_invoice_header_id = $header->internal_id;
                            $attachmnet->update();
                        }
                    }
                }
            }
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "save", $invoice_data_arr);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    public function delete($id)
    {
        try {
            $invoice_data = SupplierInvoiceData::where('internal_id', '=', $id)->get();
            foreach ($invoice_data as $data) {
                $data->delete();
            }
            $invoice_other = SupplierInvoiceOther::where('internal_id', '=', $id)->get();
            foreach ($invoice_other as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function retrieveGrnHeader($id)
    {
        try {
            $query = 'SELECT sc_grn_headers.internal_id,sc_grn_headers.trans_date,sc_grn_headers.external_id,sc_store_locations.name AS store,st_suppliers.supplier_name 
            FROM sc_grn_headers INNER JOIN sc_store_locations ON sc_grn_headers.store_id = sc_store_locations.id
            INNER JOIN st_suppliers ON sc_grn_headers.supplier_Id = st_suppliers.supplier_id WHERE  sc_grn_headers.supplier_Id = "' . $id . '" GROUP BY sc_grn_headers.internal_id';
            $grn = DB::select($query);
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "retrieveGrnHeader", $grn);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "retrieveGrnHeader", false);
        }
        return response()->json(["data" => $responseBody]);
    }



    public function retrieveGrnData($id)
    {
        try {
            $query = 'SELECT st_products.product_name,st_products.product_id,sc_grn_data.quantity,st_product_units.`name` AS unit
            FROM sc_grn_data INNER JOIN st_products ON sc_grn_data.product_id = st_products.product_primary_id
            INNER JOIN st_product_units ON sc_grn_data.unit_id = st_product_units.id
            WHERE sc_grn_data.internal_id = "' . $id . '"';

            $grn_data = DB::select($query);
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "retrieveGrnData", $grn_data);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "retrieveGrnData", false);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function setSelectedGrnHeader($id)
    {
        try {
            $query = 'SELECT sc_grn_headers.*,st_suppliers.supplier_name FROM sc_grn_headers
            LEFT JOIN st_suppliers ON sc_grn_headers.supplier_Id = st_suppliers.supplier_name
            WHERE sc_grn_headers.internal_id = "' . $id . '"';
            $grn_header = DB::select($query);
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "setSelectedGrnHeader", $grn_header[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "setSelectedGrnHeader", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function setSelectedGrn($id)
    {
        try {
            $grn = GrnHeader::find($id);
            $grn_data = GrnData::where('internal_id', '=', $id)->get();
            foreach ($grn_data as $data) {
                $product = Product::find($data->product_id);
                $data->product = $product->product_name;
                $uom = ProductUnit::find($data->unit_id);
                $uom_arr = [];
                if ($uom) {
                    array_push($uom_arr, ["value" => $uom->id, "text" => $uom->name]);
                }
                $data->quantity = ($data->quantity - ($data->transfer_quantity + $data->issues_quantity));
                $data->uom = $uom_arr;
                $data->product_primary_id = $product->product_primary_id;
                $data->product_id = $product->product_id;
            }
            $grn->data = $grn_data;
            $responseBody = $this->responseBody(true, "SupplierInvoiceController", "setSelectedGrn", $grn);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierInvoiceController", "setSelectedGrn", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }




    public function change_status(Request $request, $id)
    {

        try {

            $status = $request->get('status');
            $invoice_data_arr = $request->get("supplier_invoice_data");
            for ($i = 0; $i < count($invoice_data_arr); $i++) {
                $invoiceData = json_decode($invoice_data_arr[$i]);

                $grn_data_id = $invoiceData->grn_data_id;
                $grn_internal_id = $invoiceData->grn_internal_id;
                $grn_external_id = $invoiceData->grn_external_id;
                $grn_document_no = $invoiceData->grn_document_no;
                $qty = $invoiceData->qty;
                $value = $invoiceData->value;
                $price = $value / $qty;

                $invoice_header = SupplierInvoiceHeader::find($id);
                $invoice_header->status = $status;
                if ($invoice_header->update()) {
                    $this->addProductToSetoff($grn_data_id, $grn_internal_id, $grn_external_id, $grn_document_no, $price);
                }
            }
            $responseBody = $this->responseBody(true, "GrnHeaderController", "change_status", true);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "GrnHeaderController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    private function addProductToSetoff($grn_data_id, $grn_internal_id, $grn_external_id, $grn_document_no, $price)
    {

        $product_history = ProductHistory::where([['internal_id', '=', $grn_internal_id], ['document_no', '=', $grn_document_no]])->get();
        foreach ($product_history as $history) {
            $history_obj = ProductHistory::find($history->transaction_id);
            if ($history_obj) {
                $history_obj->cost_price = $price;
                $history_obj->update();
            }
        }

        $product_history_setoff = ProductHistorySetoff::where([['grn_internal_id', '=', $grn_internal_id], ['ref_document_no', '=', $grn_document_no]])->get();
        foreach ($product_history_setoff as $history_setoff) {
            $history_setoff_obj = ProductHistorySetoff::find($history_setoff->transaction_id);
            if ($history_setoff_obj) {
                $history_setoff_obj->cost_price = $price;
                if ($history_setoff_obj->update()) {
                    $grn_data = GrnData::find($grn_data_id);
                    if ($grn_data) {
                        $grn_data->price_updated = 1;
                        $grn_data->update();
                    }
                }
            }
        }
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
